#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/stat.h>
#ifdef HAVE_LIMITS_H
# include <limits.h>
#endif

#include "config.h"
#include "global.h"

#include "get_audio.h"
#include "tool.h"
#include "basic_io.h"

void
dist_mfbuf(float (*mfbuf)[4][576])
{
	int	i;
	short	*fr0, *fr1, *fr2;
	float	*buf_L, *buf_R;

	buf_L = mfbuf[0][0];
	buf_R = mfbuf[1][0];
	fr0 = RW.fr0;
	fr1 = RW.fr1;
	fr2 = RW.fr2;

	if(RO.mode_gr == 2){
		/* 1904(1152+BLKSIZE-224-MDCTDELAY) samples for MPEG-1 */
		if(RO.nChannel == 1){
			for(i = 0; i < 1152; i++){
				*buf_L++ = (float)(*fr0++);
			}
			for(i = 0; i < (BLKSIZE-224-MDCTDELAY)/* 752 */; i++){
				*buf_L++ = (float)(*fr1++);
			}
		}else if(RO.channels_out == 2){
			for(i = 0; i < 1152; i++){
				*buf_L++ = (float)(*fr0++);
				*buf_R++ = (float)(*fr0++);
			}
			for(i = 0; i < (BLKSIZE-224-MDCTDELAY)/* 752 */; i++){
				*buf_L++ = (float)(*fr1++);
				*buf_R++ = (float)(*fr1++);
			}
		}else{
			for(i = 0; i < 1152; i++){
				*buf_L++ = 0.5*((float)(*fr0++) + (float)(*fr0++));
			}
			for(i = 0; i < (BLKSIZE-224-MDCTDELAY)/* 752 */; i++){
				*buf_L++ = 0.5*((float)(*fr1++) + (float)(*fr1++));
			}
		}
	}else{
		/* 1328(576+BLKSIZE-224-MDCTDELAY) samples for MPEG-2 */
		if(RO.nChannel == 1){
			for(i = 0; i < 576; i++){
				*buf_L++ = (float)(*fr0++);
			}
			for(i = 0; i < 576; i++){
				*buf_L++ = (float)(*fr1++);
			}
			for(i = 0; i < (BLKSIZE-224-MDCTDELAY-576)/* 176 */; i++){
				*buf_L++ = (float)(*fr2++);
			}
		}else if(RO.channels_out == 2){
			for(i = 0; i < 576; i++){
				*buf_L++ = (float)(*fr0++);
				*buf_R++ = (float)(*fr0++);
			}
			for(i = 0; i < 576; i++){
				*buf_L++ = (float)(*fr1++);
				*buf_R++ = (float)(*fr1++);
			}
			for(i = 0; i < (BLKSIZE-224-MDCTDELAY-576)/* 176 */; i++){
				*buf_L++ = (float)(*fr2++);
				*buf_R++ = (float)(*fr2++);
			}
		}else{
			for(i = 0; i < 576; i++){
				*buf_L++ = 0.5*((float)(*fr0++) + (float)(*fr0++));
			}
			for(i = 0; i < 576; i++){
				*buf_L++ = 0.5*((float)(*fr1++) + (float)(*fr1++));
			}
			for(i = 0; i < (BLKSIZE-224-MDCTDELAY-576)/* 176 */; i++){
				*buf_L++ = 0.5*((float)(*fr2++) + (float)(*fr2++));
			}
		}
	}
}

/* The reason for
 *         int mf_samples_to_encode = ENCDELAY + 288;
 * ENCDELAY = internal encoder delay.  And then we have to add 288
 * because of the 50% MDCT overlap.  A 576 MDCT granule decodes to
 * 1152 samples.  To synthesize the 576 samples centered under this granule
 * we need the previous granule for the first 288 samples (no problem), and
 * the next granule for the next 288 samples (not possible if this is last
 * granule).  So we need to pad with 288 samples to make sure we can
 * encode the 576 samples we are interested in.
 */

void
get_audio_init(gogo_thread_data *tl)
{
	int	to_read, iread;

	RW.num_samples_read = 0;
	RW.num_samples_to_encode = ENCDELAY + 288 + 1152 /* for FFT */;
	// FFT̂߂get_audio_init()1152Tvsēǂނ̂ŁAŌ11520f

	memset(RW.sample_buf, 0, 2*2*1152*sizeof(short));
	to_read = RO.nChannel*(1152 - (ENCDELAY - MDCTDELAY));
	RW.fr0 = &RW.sample_buf[                         0];
	RW.fr1 = &RW.sample_buf[RO.nChannel*RO.framesize*1];
	RW.fr2 = &RW.sample_buf[RO.nChannel*RO.framesize*2];
	iread = readData((unsigned char *)(RW.fr1+RO.nChannel*(ENCDELAY - MDCTDELAY)), to_read*sizeof(short));
	if(iread < 0) return;/*iread;*/

	iread >>= 1;	/* iread /= sizeof(short) */
	if(RO.nChannel == 2){
		iread >>= 1;
		to_read >>=1 ;
	}
	/* subtract size of zero-filled region */
	RW.num_samples_to_encode -= (to_read-iread);

	RW.num_samples_read += iread;
}

int
get_pcm(gogo_thread_data *tl)
{
	int	to_read, iread;

	if(RW.num_samples_to_encode <= 0) return 0;

	to_read = RO.nChannel*RO.framesize;
	iread = readData((unsigned char *)RW.fr0, to_read*sizeof(short));
	if(iread < 0) return iread;	/* error */

	iread >>= 1;	/* iread /= sizeof(short) */
	if(iread != to_read) memset(RW.fr0+iread, 0, (to_read-iread)*sizeof(short));

	if(RO.mode_gr == 2){
		/* shift out */
		RW.fr2 = RW.fr0;
		RW.fr0 = RW.fr1;
		RW.fr1 = RW.fr2;
	}else{
		/* shift out */
		RW.fr3 = RW.fr0;
		RW.fr0 = RW.fr1;
		RW.fr1 = RW.fr2;
		RW.fr2 = RW.fr3;
	}

	if(RO.nChannel == 2){
		iread >>= 1;
		to_read >>=1 ;
	}
	RW.num_samples_read += iread;

	/* subtract size of zero-filled region */
	RW.num_samples_to_encode -= (to_read-iread);

	return to_read;
}

