#ifndef GLOBAL_H_
#define GLOBAL_H_

#include "common.h"
#include "gogo.h"
#include "l3side.h"
#include "thread.h"

#ifdef _GOGO_C_
	#define EXT
#else
	#define EXT extern
#endif

enum vbr_mode_e {
  vbr_off=0,
  vbr_rh,
  vbr_abr,
  vbr_default=vbr_rh  /* change this to change the default VBR mode of LAME */ 
};


/**
 *  ATH related stuff, if something new ATH related has to be added,
 *  please plugg it here into the ATH_t struct
 */
typedef struct ath_t
{
    int     use_adjust;     // do we want to use the auto adjustment yes/no
    FLOAT8  adjust;         // lowering based on peak volume, 1 = no lowering
    FLOAT8  adjust_limit;   // limit for dynamic ATH adjust
    FLOAT8  decay;          // determined to lower x dB each second
    FLOAT8  l[SBMAX_l];     // ATH for sfbs in long blocks
    FLOAT8  s[SBMAX_s];     // ATH for sfbs in short blocks
    FLOAT8  cb[CBANDS];     // ATH for convolution bands
} ATH_t;


/**
 * "bit_stream.h" Type Definitions 
 */
typedef struct  bit_stream_struc {
    unsigned char *buf;         /* bit stream buffer */
    int         buf_size;       /* size of buffer (in number of bytes) */
    int         totbit;         /* bit counter of bit stream */
    int         buf_byte_idx;   /* pointer to top byte in buffer */
    int         buf_bit_idx;    /* pointer to top bit of top byte in buffer */
    
    /* format of file in rd mode (BINARY/ASCII) */
} Bit_stream_struc;

/**
 *  VBR_seek_info_t
 */
typedef struct vbr_seek_info_t
{
    int sum;    // what we have seen so far
    int seen;   // how many frames we have seen in this chunk
    int want;   // how many frames we want to collect into one chunk
    int pos;    // actual position in our bag
    int size;   // size of our bag
    int bag[400];   // bag
} VBR_seek_info_t;

#define MAX_HEADER_BUF 256
#define MAX_HEADER_LEN 40 /* max size of header is 38 */
/**
 * variables for bitstream.c 
 * mpeg1: buffer=511 bytes  smallest frame: 96-38(sideinfo)=58
 * max number of frames in reservoir:  8 
 * mpeg2: buffer=255 bytes.  smallest frame: 24-23bytes=1
 * with VBR, if you are encoding all silence, it is possible to
 * have 8kbs/24khz frames with 1byte of data each, which means we need
 * to buffer up to 255 headers!
 * also, max_header_buf has to be a power of two */
typedef struct header_buf_t {
    int write_timing;
    int ptr;
    char buf[MAX_HEADER_LEN];
} Header_buf_t;


/**
 * the following variables are constant(Read Only) in encoding 
 * Don't change them except in gogo.c 
 * ̕ϐ̓GR[h͒萔ł 
 * gogo.cȊOł͏ł͂Ȃ 
 */
typedef struct ro_t {
	/* internal variables */
	int nChannel;
	int inpFreqHz;
//	int encFreqHz;
//	int outFreqHz;
	int mode_gr;
	int is_mpeg1; /* 1 for MPEG-1, 0 for MPEG-2(.5) */
	int nSample;
	int rateKbps;
	int emphasis;
	int quality;	/* 2:slow, 5:default, 7:fast */
	int mode;		/* mode = STEREO, JOINT_STEREO, MONO */
	int framesize;
	int version;                    /* 0=MPEG-2  1=MPEG-1  (2=MPEG-2.5)     */
        int copyright;          /* mark as copyright. default=0           */
        int original;           /* mark as original. default=1            */
        int out_samplerate;         /* output_samp_rate.
                                         default: LAME picks best value 
                                         at least not used for MP3 decoding:
                                         Remember 44.1 kHz MP3s and AC97           */

  /*
   * psycho acoustics and other arguments which you should not change 
   * unless you know what you are doing
   */
	int ATHtype;                    /* select ATH formula                   */
        /*
         * set either brate>0  or compression_ratio>0, LAME will compute
         * the value of the variable not set.
         * Default is compression_ratio = 11.025
         */
        int brate;              /* bitrate                                    */
        /* frame params */
        int padding_type;       /* 0=no padding, 1=always pad,
                                   2=adjust padding, default=2            */

        /* VBR control */
        int VBR;
        int VBR_q;
        int VBR_min_bitrate_kbps;
        int VBR_max_bitrate_kbps;




	/* CPU */
	int unit;
	int nCPU;

	/* Thread */
	int	nThread;
	gogo_thread_data	*tl;

	/* file */
	char inFileName[MAX_FILE_LEN];
	char outFileName[MAX_FILE_LEN];
	struct MCP_INPDEV_USERFUNC userIn;	/* if user.pUserFunc == NULL then using file I/O */
	MPGE_USERFUNC userOut;
	int useVbrTagWithUserOut;

	/* etc */
	int debug;
	int silent;
	void (*printf)(const char *, ...);

	// ͕̂KvȂ́H --> rbg[gɉătB^̃JbgItlݒ肷Kv̂ŗv 
        /* resampling and filtering */
        int lowpassfreq;        /* freq in Hz. 0=gogo choses.  -1=no filter  */
        int highpassfreq;       /* freq in Hz. 0=gogo choses.  -1=no filter  */
        int lowpasswidth;       /* freq width of filter, in Hz (default=15%) */
        int highpasswidth;      /* freq width of filter, in Hz (default=15%) */

	// ȍ~C̍\̂I܂łɂ͈̂ȑO RW ɂϐBł
	// align Kvgppx߂͍̂̂\̂̐擪߂Ɉڂ\

	// ̂̓GR[hJnɈxω邾Ȃ̂ RO ɂĂ܂
	// ܂CR[h𐮗΁C炭svɂȂ܂D
        int lame_encode_frame_init;     
        int iteration_init_init;
	// int fill_buffer_resample_init; // ݖgp resampling ɕ邩
        int psymodel_init;

	int	channels_out;	/* number of channels in the output data stream */ // RO
  int VBR_min_bitrate;            /* min bitrate index */ // RO
  int VBR_max_bitrate;            /* max bitrate index */ // RO
  int CBR_bitrate;
  int samplerate_index;  // RO

  /* lowpass and highpass filter control */
  float lowpass1,lowpass2;        /* normalized frequency bounds of passband */  // RO
  float highpass1,highpass2;      /* normalized frequency bounds of passband */  // RO

  /* polyphase filter */
  int lowpass_band;          /* zero bands >= lowpass_band in the polyphase filterbank */  // RO
  int highpass_band;         /* zero bands <= highpass_band */  // RO
  int lowpass_start_band;    /* amplify bands between start */  // RO
  int lowpass_end_band;      /* and end for lowpass */ // RO
  int highpass_start_band;   /* amplify bands between start */ // RO
  int highpass_end_band;     /* and end for highpass */ // RO

  int noise_shaping;        /* 0 = none 
                               1 = ISO AAC model
                               2 = allow scalefac_select=1  
                             */ // RO

  int noise_shaping_amp;    /*  0 = ISO model: amplify all distorted bands
                                1 = amplify only most distorted band
                                2 = amplify bands using? 
                                3 = amplify bands using?
			     */ // RO

  int noise_shaping_stop;   /* 0 = stop at over=0, all scalefacs amplified or
                               a scalefac has reached max value
                               1 = stop when all scalefacs amplified or        
                                   a scalefac has reached max value
                               2 = stop when all scalefacs amplified 
			    */ // RO

  int use_best_huffman;     /* 0 = no.  1=outside loop  2=inside loop(slow) */
                            //RO
  /* used for padding */
  int frac_SpF; // RO

  /* variables used by quantize.c */
  FLOAT8 decay; // RO

  int sideinfo_len; // RO

  FLOAT8 amp_lowpass[32]; // RO
  FLOAT8 amp_highpass[32]; // RO

  scalefac_struct scalefac_band; // RO

  /* DATA FROM PSYMODEL.C */
/* The static variables "r", "phi_sav", "new", "old" and "oldest" have    */
/* to be remembered for the unpredictability measure.  For "r" and        */
/* "phi_sav", the first index from the left is the channel select and     */
/* the second index is the "age" of the data.                             */
  FLOAT8	minval[CBANDS];                    // RO
  FLOAT8 s3_s[CBANDS][CBANDS];                     // RO   
  FLOAT8 s3_l[CBANDS][CBANDS];                     // RO

  /* unpredictability calculation
   */
  int cw_upper_index; // RO
  int cw_lower_index; // RO ĂCǂȃGR[hł == 6
  /* fft.c    */
  FLOAT window[BLKSIZE], window_s[BLKSIZE_s/2]; // RO
  
  /* Scale Factor Bands    */
  FLOAT8	w1_l[SBMAX_l], w2_l[SBMAX_l]; // RO
  FLOAT8	w1_s[SBMAX_s], w2_s[SBMAX_s]; // RO
  FLOAT8 mld_l[SBMAX_l],mld_s[SBMAX_s]; // RO
  int	bu_l[SBMAX_l],bo_l[SBMAX_l] ; // RO
  int	bu_s[SBMAX_s],bo_s[SBMAX_s] ; // RO
  int	npart_l,npart_s; // RO
  int	npart_l_orig,npart_s_orig; // RO
  
  int	s3ind[CBANDS][2];  // RO
  int	s3ind_s[CBANDS][2]; // RO
  FLOAT8 SNR_s[CBANDS]; // RO

  int	numlines_s[CBANDS]; // RO
  int	numlines_l[CBANDS]; // RO

  ATH_t	ATH;   // all ATH related stuff // RO VBRRW`

  /* VBR tag */
  int nZeroStreamSize;
  int TotalFrameSize;
} RO_t;
EXT RO_t RO;

/**
 * ̕ϐ̓GR[h͕ϐł 
 * }`XbhGR[ĥƂ͋Lɒuׂϐł 
 * ACgLbVɒ 
 */
typedef struct rw_t {
	/* variables for newmdct.c */
	float	subband_buf[2][18][SBLIMIT];	/* PFB -> MDCT(shared part) */

	short	sample_buf[2*2*1152];	/* sample input -> shiftin(shared part) */
	short	*fr0, *fr1, *fr2, *fr3; // shared

  /*  
   * Some remarks to the Class_ID field:
   * The Class ID is an Identifier for a pointer to this struct.
   * It is very unlikely that a pointer to lame_global_flags has the same 32 bits 
   * in it's structure (large and other special properties, for instance prime).
   *
   * To test that the structure is right and initialized, use:
   *     if ( gfc -> Class_ID == LAME_ID ) ...
   * Other remark:
   *     If you set a flag to 0 for uninit data and 1 for init data, the right test
   *     should be "if (flag == 1)" and NOT "if (flag)". Unintended modification
   *     of this element will be otherwise misinterpreted as an init.
   */
  
	unsigned int	num_samples_read; // shared
	int		num_samples_to_encode;  // shared

	int frameNum;                   /* number of frames encoded             */ // shared
	int totalframes; // shared

	/* psycho acoustics */
	float	tot_ener[4];	// shared
  FLOAT8 ms_ratio[2];  // shared
  /* used for padding */
  int slot_lag; // shared(modefied in lame_encode_mp3_frame()) rOK

  /* variables used by quantize.c */
  int OldValue[2]; // shared	quantize.c
  int CurrentStep; // shared	quantize.c

  char bv_scf[576]; // shared	takehiro.c
  
  int sfb21_extra; /* will be set in lame_init_params */  // shared	quantize.c, quantize_pvt.c
  
  Header_buf_t header[MAX_HEADER_BUF]; // shared

  int h_ptr; // shared
  int w_ptr; // shared
  int ancillary_flag; // shared
  

// ܂ŔrOK
/* ߂߂
	ResvSize }Y
*/

	Bit_stream_struc   bs;  // shared rOK
  int ResvSize; /* in bits */ // shared	ResvFrameBegin(), ResvMaxBits()ŎQƁAResvAdjust(), ResvFrameEnd(), bitstream.cŕύX
  int ResvMax;  /* in bits */ // shared rOK(mutex_mp3out, mutex_l3encŔr邱)
	int	main_data_begin;	// shared rOK(mutex_mp3out, mutex_l3encŔr邱)

  /* DATA FROM PSYMODEL.C */
/* The static variables "r", "phi_sav", "new", "old" and "oldest" have    */
/* to be remembered for the unpredictability measure.  For "r" and        */
/* "phi_sav", the first index from the left is the channel select and     */
/* the second index is the "age" of the data.                             */
  FLOAT8	nb_1[4][CBANDS], nb_2[4][CBANDS];  // shared rOK

  III_psy_xmin thm[4]; // shared rOK
  III_psy_xmin en[4];  // shared rOK
  
  /* unpredictability calculation
   */
  FLOAT ax_sav[4][2][HBLKSIZE]; // shared rOK
  FLOAT bx_sav[4][2][HBLKSIZE]; // shared rOK
  FLOAT rx_sav[4][2][HBLKSIZE]; // shared rOK
  FLOAT cw[HBLKSIZE]; // shared rOK

  /* ratios  */
  FLOAT8 pe[4]; // shared rOK
  FLOAT8 ms_ratio_s_old,ms_ratio_l_old; // shared rOK

  /* block type */
  int	blocktype_old[2]; // shared rOK

  /* VBR tag */
  VBR_seek_info_t VBR_seek_table; 
  int nVbrNumFrames;

#ifdef	MT_ENCODER
	/* mutex */
#define	MAX_CRITICAL_REGION	8
	gogo_mutex	mutex[MAX_CRITICAL_REGION];
#endif
} RW_t;
EXT RW_t RW;

#endif /* GLOBAL_H_ */
