#define _GOGO_C_
#include <assert.h>

#include "config.h"
#include "global.h"
#include "version.h"

#include "lame.h"
#include "get_audio.h"
#include "cpu.h"
#include "fft.h"
#include "tool.h"
#include "basic_io.h"
#include "util.h"
#include "newmdct.h"
#include "psymodel.h"
#include "quantize.h"
#include "quantize_pvt.h"
#include "bitstream.h"
#include "vbrtag.h"
#include "reservoir.h"

/*
 *	R[h̗
 *
 *	1) MPGE_initializeWork()
 *		1. RO  zero clear
 *		2. lame_init()
 *		3. RȌ
 *
 *	2) MPGE_setConfigure()
 *		ʂRO
 *
 *	3) MPGE_detectConfigure()
 *		1. initFileIO();
 *				> nChannel, inpFreqHz, nSample, outFileName
 *		2. lame_init_params()
 */
MERET EXPORT
MPGE_initializeWork(void)
{

	/* setup Read Only global variables */
	memset(&RO, 0, sizeof(RO));
	lame_init();

	RO.nChannel = 2;
	RO.inpFreqHz = 44100;
	RO.out_samplerate = 0;	/* RO.encFreqHz = 0; 0=auto configure */
	RO.nSample = -1;
	RO.rateKbps = 128;
	RO.quality = 5;
	RO.mode = NOT_SET;

	RO.debug = 0;
	RO.silent = 0;
	RO.printf = errPrintf;

	/* setup CPU */
	initCPU();
	RO.unit = haveUNIT();
#if 0
	RO.nCPU = getNumOfCPU();
#endif
	RO.nCPU = RO.nThread = 0;	/* auto configure */
	if(gogo_get_cpu_count(&RO.nCPU, &RO.nThread)) return ME_INTERNALERROR;
	if(RO.nThread > 1) RO.unit |= MU_tMULTI;
#if defined(MT_ENCODER)
	if(gogo_intialize_thread_unit()) return ME_INTERNALERROR;
#endif
#if defined(CPU_I386)
	if((RO.unit & MU_tFPU) == 0) return ME_NOFPU;
#elif defined(CPU_PPC)
	if((RO.unit & MU_tPPC) == 0) return ME_NOFPU;
#else
	#error "unknown CPU"
#endif

#if defined(WIN32)
//  gogo_intialize_thread_unit()ɈڂقƎvB
	MPGE_setConfigure(MC_THREAD_PRIORITY, THREAD_PRIORITY_HIGHEST, 0);
#endif
	setupUNIT( RO.unit );
	return ME_NOERR;
} /* MPGE_initializeWork */

MERET EXPORT
MPGE_getUnitStates(unsigned long *pUnit)
{
	*pUnit = haveUNIT();
	return ME_NOERR;
} /* MPGE_getUnitStates */

MERET EXPORT
MPGE_endCoder(void)
{
//	if(pid3tag) mem_free( (void **)&pid3tag );
//	nid3taglen = 0;
//	pid3tag = NULL;

//	if(pRiffInfos) mem_free( (void **)&pRiffInfos );

//	riffInfosLen = 0;
//	pRiffInfos = NULL;

#if defined(MT_ENCODER)
	if(gogo_finalize_thread_unit()) return ME_INTERNALERROR;
#endif
	return ME_NOERR;
} /* MPGE_endCoder */

MERET EXPORT
MPGE_setConfigure(UPARAM mode, UPARAM dwPara1, UPARAM dwPara2)
{

	switch(mode){
	case MC_INPUTFILE: /* OK */
		if(dwPara1 == MC_INPDEV_FILE){
			RO.userIn.pUserFunc = MPGE_NULL_FUNC;
			if(strlen((char *)dwPara2) >= MAX_FILE_LEN) return ME_PARAMERROR;
			strcpy(RO.inFileName, (char *)dwPara2);
			return ME_NOERR;
		}else
		if(dwPara1 == MC_INPDEV_STDIO){
			RO.userIn.pUserFunc = MPGE_NULL_FUNC;
			strcpy(RO.inFileName, "stdin");
			return ME_NOERR;
		}else
		if(dwPara1 == MC_INPDEV_USERFUNC){
			RO.userIn = *((struct MCP_INPDEV_USERFUNC *)dwPara2);
			RO.inFileName[0] = '\0';
			return ME_NOERR;
		}
		return ME_PARAMERROR;

	case MC_OUTPUTFILE: /* OK */
		if(dwPara1 == MC_OUTDEV_FILE){
			RO.userOut = MPGE_NULL_FUNC;
			RO.useVbrTagWithUserOut = FALSE;
			if(strlen((char *)dwPara2) >= MAX_FILE_LEN) return ME_PARAMERROR;
			strcpy(RO.outFileName, (char *)dwPara2);
			return ME_NOERR;
		}else
		if(dwPara1 == MC_OUTDEV_USERFUNC){
			RO.userOut = (MPGE_USERFUNC)dwPara2;
			RO.useVbrTagWithUserOut = FALSE;
			return ME_NOERR;
		}else
		if(dwPara1 == MC_OUTDEV_USERFUNC_WITHVBRTAG){
			RO.userOut = (MPGE_USERFUNC)dwPara2;
			RO.useVbrTagWithUserOut = TRUE;
			return ME_NOERR;
		}
		return ME_PARAMERROR;

	case MC_ENCODEMODE: /* OK */
		switch(dwPara1){
		case MC_MODE_MONO:
			RO.mode = MONO;
			break;
		case MC_MODE_STEREO:
			RO.mode = STEREO;
			break;
		case MC_MODE_JOINT:
			RO.mode = JOINT_STEREO;
			break;
		default:
			return ME_PARAMERROR;
		}
		return ME_NOERR;

	case MC_BITRATE:
		RO.rateKbps = dwPara1;
		return ME_NOERR;

	case MC_INPFREQ: /* OK */
		RO.inpFreqHz = dwPara1;
		return ME_NOERR;

	case MC_OUTFREQ: /* OK */
		RO.out_samplerate = dwPara1;	/* RO.encFreqHz = dwPara1; */
		return ME_NOERR;

	case MC_EMPHASIS: /* OK */
		RO.emphasis = dwPara1;
		return ME_NOERR;

	case MC_USEPSY: /* OK */
//			fast_mode = dwPara1 ? FALSE: TRUE ;
		RO.quality = dwPara1 ? 2 : 8;
		return ME_NOERR;

	case MC_ENCODE_QUALITY: /* OK */
		if(dwPara1 < 1 || dwPara1 > 9) return ME_PARAMERROR;
		RO.quality = dwPara1;
		return ME_NOERR;

/* eXg */
		case MC_ABR:
			RO.VBR = vbr_abr;
			return ME_NOERR;
		case MC_CPU:
			RO.nCPU = dwPara1; RO.nThread = 0;
			if(RO.nCPU < 0) return ME_PARAMERROR;
			RO.unit &= ~MU_tMULTI;
			gogo_get_cpu_count(&RO.nCPU, &RO.nThread);
			if(RO.nThread > 1) RO.unit |= MU_tMULTI;
			return ME_NOERR;
/* ܈ڐA */
		case MC_VBR:
			RO.VBR = vbr_default;
			RO.VBR_q = dwPara1;
			if (RO.VBR_q <0) RO.VBR_q=0;
			if (RO.VBR_q >9) RO.VBR_q=9;
			return ME_NOERR;
		case MC_VBRBITRATE:
//			VBR_min_rate_idx = get_rate_idx(dwPara1, 1); 
//			VBR_max_rate_idx = get_rate_idx(dwPara2, 1);
//			if(dwPara1 > dwPara2) return ME_PARAMERROR;
//			if( VBR_min_rate_idx == -1 || VBR_max_rate_idx == -1 ) return ME_PARAMERROR;
			RO.brate = dwPara1;
			RO.VBR_min_bitrate_kbps=RO.brate;
			if((int)dwPara2 != -1) RO.VBR_max_bitrate_kbps=dwPara2;
			return ME_NOERR;
		return ME_NOERR;
	}/* switch(mode) */
	return ME_PARAMERROR;
} /* MPGE_setConfigure */

/* open file and read wave header */
static MERET
initFileIO(void)
{
	unsigned int size;
	int freq, bit, channel;

	/****   input device   ***/
	initBasicIO();
	if(RO.userIn.pUserFunc){
		size = RO.userIn.nSize;
		bit = RO.userIn.nBit;
		channel = RO.userIn.nChn;
		freq = RO.userIn.nFreq;
	}else{
		if(openInputFile(RO.inFileName) == ERR) return ME_INFILE_NOFOUND;

		/* analyze wav-header */
		if(getWaveInfo(&size, &bit, &freq, &channel) == ERR){
			closeInputFile();
			return ME_WAVETYPE_ERR;
		}
	}
	if(bit != 16 || (channel != 1 && channel != 2)){
		return ME_WAVETYPE_ERR;
	}

	RO.nChannel = channel;
	RO.inpFreqHz = freq;
	RO.nSample = size / (channel * bit / 8);

	/***   output device   ***/
	if(RO.userOut){
//		RO.printf("not supported user out function in initFileIO\n");
//		exit(1);
	}else{
		/* ]vȂbۂǌ݊̂ */
		if(!RO.outFileName[0]){
			char name[MAX_FILE_LEN];
			if(changeSuffix(name, RO.inFileName, MAX_FILE_LEN) == ERR) return ME_INTERNALERROR;
			strcpy(RO.outFileName, name);
		}

		if(openOutputFile(RO.outFileName) == ERR) return ME_OUTFILE_NOFOUND;
	}

	return ME_NOERR;
} /* initFileIO */

static int
initializeThread(void)
{
	int	i;
	gogo_thread_data	*aligned;
	void	*unaligned;

	assert(RO.nThread >= 1);
	RO.tl = NULL;

	assert(RO.nThread >= 1);
	for(i = 0; i < RO.nThread; i++){
		unaligned = malloc(sizeof(gogo_thread_data) + 16);
		if(unaligned == NULL) return ME_NOMEMORY;

		/* align to 16 byte boundary */
		aligned = (void *)(((int)unaligned + 15) & -16);

		/* initialize */
		memset(aligned, 0, sizeof(gogo_thread_data));
		aligned->tid = i;
		aligned->unaligned = unaligned;
		aligned->sb[0] = aligned->sb_sample[0];
		aligned->sb[1] = aligned->sb_sample[1];
		aligned->sb[2] = aligned->sb_sample[2];

		aligned->bitrate_index = RO.CBR_bitrate;

		aligned->next = RO.tl;
		RO.tl = aligned;
	}
	return ME_NOERR;
}

MERET EXPORT
MPGE_detectConfigure(void)
{
	MERET ret;

	ret = initFileIO();
	if(ret != ME_NOERR) return ret;

	// gogo cfg
	RO.brate = RO.rateKbps;
	RO.out_samplerate = RO.inpFreqHz;	// TvO[gRo[^[
						// Ƃ肠Ă // RO.encFreqHz;
						// encFreqHz͕svƎvB
						// out_samplerate != inpFreqHz ̂ƂɃTvO[gRo[^[NƂƂŁB
	lame_init_params();

	ret = initializeThread();
	if(ret != ME_NOERR) return ret;

	return ME_NOERR;
} /* MPGE_detectConfigure */

/* `̊֌W encoder.h ɒu񂩂(^^; */
void encodeframe_init(gogo_thread_data *tl);
int lame_encode_mp3_frame(gogo_thread_data *tl);
#if defined(MT_ENCODER)
void encodetrack_init(gogo_thread_data *tl);
int lame_encode_mp3_frame_multi(gogo_thread_data *tl);
#endif

static int
encodeframe(void)
{
	int iread, imp3;
	gogo_thread_data	*tl = RO.tl;

	if (RO.lame_encode_frame_init==0 )  {
		RO.lame_encode_frame_init=1;

		get_audio_init(tl);
		dist_mfbuf(tl->mfbuf);
		encodeframe_init(tl);
	}
	iread = get_pcm(tl);
	if(iread < 0) return  -1 /* iread */;
	if(iread == 0){
                /* mp3 related stuff.  bit buffer might still contain some mp3 data */
                flush_bitstream(tl);
	}else{
		dist_mfbuf(tl->mfbuf);
		imp3 = lame_encode_mp3_frame(tl);
		if(imp3 < 0){
			RO.printf("mp3 internal error:  error code=%i\n",imp3);
			return -1;
		}
	}

	if( put_mp3( &RW.bs ) < 0 ) return -1;
	return iread;
} /* encodeframe */

MERET EXPORT
MPGE_processFrame(void)
{
	int ret;
	ret = encodeframe();
	if(ret == -1){
		return ME_INTERNALERROR;
	}else
	if(ret == 0){
		return ME_EMPTYSTREAM;
	}
	return ME_NOERR;
} /* MPGE_processFrame */

#if defined(MT_ENCODER)
static void *
encodetrack(gogo_thread_data *tl)
{
	int iread, imp3;

	tl->exit_status = ME_NOERR;

	while(1){
		tl->critical_region = RW.mutex;
		if((tl->exit_status = gogo_lock_mutex(tl->critical_region)) != 0){
			return NULL;
		}
		iread = get_pcm(tl);
		if(iread <= 0){
			tl->exit_status = iread;
			break;
		}
		dist_mfbuf(tl->mfbuf);
		tl->exit_status = gogo_lock_mutex(tl->critical_region+1);
		if(tl->exit_status) break;
		tl->exit_status = gogo_unlock_mutex(tl->critical_region++);
		if(tl->exit_status) break;

		imp3 = lame_encode_mp3_frame_multi(tl);
		if(imp3 < 0){
			RO.printf("thread %d: mp3 internal error:  error code=%i\n",imp3, tl->tid);
			tl->exit_status = -1;
			break;
		}

		if( put_mp3( &RW.bs ) < 0 ){
			tl->exit_status = -1;
			break;
		}

		tl->exit_status = gogo_unlock_mutex(tl->critical_region);
		if(tl->exit_status) return NULL;
	}
	gogo_unlock_mutex(tl->critical_region);
	return NULL;
}
#endif

MERET EXPORT
MPGE_processTrack(void)
{
	int ret;
#if defined(MT_ENCODER)
	if(RO.nThread > 1){
		/* mulithread encoding */
		gogo_thread_data	*tl;
		int i;

		for(i= 0; i < MAX_CRITICAL_REGION; i++){
			gogo_create_mutex(&RW.mutex[i]);
		}

		get_audio_init(RO.tl);
		dist_mfbuf(RO.tl->mfbuf);
		encodetrack_init(RO.tl);
		for(tl = RO.tl->next; tl != NULL; tl = tl->next){
			ret = gogo_create_thread(&tl->thread, encodetrack, tl);
			if(ret) return ret;
		}

		encodetrack(RO.tl);
		ret = RO.tl->exit_status;

		for(tl = RO.tl->next; tl != NULL; tl = tl->next){
			gogo_join_thread(&tl->thread);
			gogo_destroy_thread(&tl->thread);
			if(tl->exit_status < 0) ret = tl->exit_status;
		}

// ŏIt[o͂l͒Nł傤H RO.tln̂̓}Y(^^;;;
// ŏIt[RO.tl->paddingKvȂ̂B
                /* mp3 related stuff.  bit buffer might still contain some mp3 data */
                flush_bitstream(RO.tl);

		if( put_mp3( &RW.bs ) < 0 ){
			return ME_INTERNALERROR;
		}

		for(i= 0; i < MAX_CRITICAL_REGION; i++){
			gogo_destroy_mutex(&RW.mutex[i]);
		}
		return ret;
	}
#endif
	while((ret = encodeframe()) > 0);
	if(ret == -1){
		return ME_INTERNALERROR;
	}
	return ME_NOERR;
} /* MPGE_processTrack */

static int
finalizeThread(void)
{
	gogo_thread_data	*aligned;
	void	*unaligned;

	do{
		aligned = RO.tl;
		RO.tl = aligned->next;
		unaligned = aligned->unaligned;
		free(unaligned);
	}while(RO.tl != NULL);
	return ME_NOERR;
}

MERET EXPORT
MPGE_closeCoder(void)
{
	int ret;

	/* add VBR tags to mp3 file */
	lame_mp3_tags_fid();

	closeInputFile();
	closeOutputFile();
	ret = finalizeThread();
	if(ret != ME_NOERR) return ret;
	return ME_NOERR;
} /* MPGE_closeCoder */

MERET EXPORT
MPGE_getConfigure(UPARAM mode, void *param )
{
	int		*iparam = (int *)param;

	// GR[hJn܂Őݒ擾ĂяoȂ
//	if( !musicin )
//		return ME_PARAMERROR;

	switch( mode ){
		case MG_INPUTFILE:
			strcpy( param, RO.inFileName );
			return ME_NOERR;
		case MG_OUTPUTFILE:
			strcpy( param, RO.outFileName );
			return ME_NOERR;
		case MG_ENCODEMODE:
			switch( RO.mode ){
				case MONO:
					*iparam = MC_MODE_MONO;
					return ME_NOERR;
				case STEREO:
					*iparam = MC_MODE_STEREO;
					return ME_NOERR;
				case JOINT_STEREO:
					*iparam = MC_MODE_JOINT;
					return ME_NOERR;
			}
			return ME_PARAMERROR;
		case MG_BITRATE:
			*iparam = RO.rateKbps;
			return ME_NOERR;
		case MG_INPFREQ:
			*iparam = RO.inpFreqHz;
			return ME_NOERR;
		case MG_OUTFREQ:
			*iparam = RO.out_samplerate;	/* RO.encFreqHz; */
			return ME_NOERR;
		case MG_STARTOFFSET:
			*iparam = -1;
			return ME_NOERR;
		case MG_USEPSY:
			*iparam = RO.quality <= 5 ? TRUE : FALSE;
			return ME_NOERR;
		case MG_USEMMX:
			*iparam = FALSE;	//( useUNIT & tMMX ) ? TRUE : FALSE;
			return ME_NOERR;
		case MG_USE3DNOW:
			*iparam = FALSE;	//( useUNIT & t3DN ) ? TRUE : FALSE;
			return ME_NOERR;
		case MG_USEKNI:
			*iparam = FALSE;	//( useUNIT & tSSE ) ? TRUE : FALSE;
			return ME_NOERR;
		case MG_USEE3DNOW:
			*iparam = FALSE;	//( useUNIT & tE3DN ) ? TRUE : FALSE;
			return ME_NOERR;
		case MG_USESPC1:
			*iparam = FALSE;	//( useUNIT & tSPC1 ) ? TRUE : FALSE;
			return ME_NOERR;
		case MG_USESPC2:
			*iparam = FALSE;	//( useUNIT & tSPC2 ) ? TRUE : FALSE;
			return ME_NOERR;
		case MG_COUNT_FRAME:
			*iparam = RW.totalframes;		//totalframes;
			return ME_NOERR;
		case MG_NUM_OF_SAMPLES:
			*iparam = RO.framesize;		//gl.frameSize;
			return ME_NOERR;
		case MG_MPEG_VERSION:
			*iparam = 0;		//gl.version;
			return ME_NOERR;
#if	defined(USE_BTHREAD)
		case MG_READTHREAD_PRIORITY:
			*iparam = intReadThreadPriority;
			return ME_NOERR;
#endif
		case MG_FRAME:
			*iparam = RW.frameNum;
			return ME_NOERR;
	}
	return ME_PARAMERROR;
}

MERET EXPORT
MPGE_getVersion( unsigned long *vercode,  char *verstring )
{
	*vercode = VERSION_NUM;
	strcpy( verstring, VERSION );

	return ME_NOERR;
}

/* 53Kclk@PIII */
static void pow075_C(float xr[576], float xrpow[576], float *psum, float *pmax, uint32 *xr_sign)
{
        float   srpow_sum = 0.0, xrpow_max = 0.0, tmp;
        int     i;
        for (i = 0; i < 576; ++i) {
//				xr_sign[i] = (xr[i] < 0) ? 0xFFFFFFFF : 0;
				xr_sign[i] = (*(int *)&xr[i]) >> 31;
                tmp = xr[i] = fabs (xr[i]);	// xr_signɓĂ
                srpow_sum += tmp;
                xrpow[i] = sqrt (tmp * sqrt(tmp));
                if(xrpow_max < xrpow[i]) xrpow_max = xrpow[i];
        }
        *psum = srpow_sum;
        *pmax = xrpow_max;
}

#ifdef CPU_I386
static void (*pow075)(float xr[576], float xrpow[576], float *psum, float *pmax, uint32 *xr_sign);
void pow075_SSE(float xr[576], float xrpow[576], float *psum, float *pmax, uint32 *xr_sign);

void setup_pow075( int unit )
{
	if( unit & MU_tSSE ){
		pow075 = pow075_SSE;
	}else
	{
		pow075 = pow075_C;
	}
}
#else
#define pow075 pow075_C
#endif

#include "encoder.c"
#if defined(MT_ENCODER)
#  undef MT_ENCODER
#  define ST_ENCODER
#include "encoder.c"
#endif
