#include <setjmp.h>
#include <signal.h>

#if defined(USE_PTHREAD)
	#include <pthread.h>
	#include <semaphore.h>
	#if defined(__FreeBSD__)
		#include <sys/sysctl.h>
	#endif
#endif
#ifdef	WIN32
	#include <windows.h>
	#include <winbase.h>
	#include <process.h>
#endif
#ifdef USE_OS2THREAD
	#define INCL_DOS
	#include <os2.h>
#endif
#ifdef USE_BTHREAD
	#include <OS.h>
#endif

#if defined(__FreeBSD__)
	#include <floatingpoint.h>
#endif
#ifdef ABORTFP
	#include <fpu_control.h>
#endif

#include "common.h"
#include "cpu.h"
#include "gogo.h"
#include "global.h"

/*********************************************************************/

static jmp_buf jmpPtr;

static void
SSE_not_support(int sig)
{
	signal(SIGILL, SIG_DFL);
	longjmp(jmpPtr, 1);
} /* SSE_not_support */

extern int haveUNITa(void); /* defined in cpua.nas */
extern void setPIII_round(void);

int
haveUNIT(void)
{
	static int unit = -1;

	if(unit != -1) return unit;
	unit = haveUNITa();

#if 0
	if(getNumOfCPU() > 1) unit |= MU_tMULTI;	// gogo.cֈړ
#endif
#if defined(WIN32) //&& defined(_WINDOWS) /* for only WinGUI */
	if(unit & MU_tSSE){
		__try{
			setPIII_round();
		}
		__except(EXCEPTION_EXECUTE_HANDLER){
			unit &= ~(MU_tSSE|MU_tSSE2);
		}
	}
#elif defined(__unix__) || defined(WIN32) || defined(__os2__) || defined(__dos__)
	if(unit & MU_tSSE){
		MERET ret;
		ret = setjmp(jmpPtr);
		if(!ret){
			signal(SIGILL, SSE_not_support);
			setPIII_round();
			signal(SIGILL, SIG_DFL);
		}else{
			unit &= ~(MU_tSSE|MU_tSSE2);
		}
	}
#else
	if(unit & MU_tSSE) setPIII_round();
#endif
	return unit;
} /* haveUNIT */

#if 0
int
getNumOfCPU(void)
{

#if defined(__linux__)

	FILE *cpuinfo;
	char buffer[256];
	int nCPU = 0;
	if((cpuinfo = fopen("/proc/cpuinfo", "r")) != NULL){
		while(fgets(buffer, sizeof buffer, cpuinfo) != NULL){
			if(memcmp(buffer, "processor", (sizeof "processor") - 1) == 0){
				nCPU++;
			}
		}
		fclose(cpuinfo);
	}
	/* CPU number detection for Linux emulator on FreeBSD */
	else if((cpuinfo = popen("/sbin/sysctl hw.ncpu", "r")) != NULL){
		if(fgets(buffer, sizeof buffer, cpuinfo) != NULL){
			int	cpu;

			if(sscanf(buffer, "hw.ncpu: %d", &cpu) == 1){
				nCPU = cpu;
			}
		}
		fclose(cpuinfo);
	}
	return nCPU;

#elif defined(__FreeBSD__)

	int	sysname[] = {CTL_HW, HW_NCPU}, len, cpu;
	int nCPU = 1;
	len = sizeof(int);
	if(sysctl(sysname, 2, &cpu, &len, NULL, 0) == 0){
		if(len == sizeof(int)) nCPU = cpu;
	}
	return nCPU;

#elif defined(BeOS)

	system_info udtsystem_info;
	get_system_info(&udtsystem_info);
	return udtsystem_info.cpu_count;

#elif defined(__os2__)

	const int QSV_NUMPROCESSORS = 26;
	APIRET apiretRc = 0;
	int nCPU = 1;
	apiretRc = DosQuerySysInfo(QSV_NUMPROCESSORS, QSV_NUMPROCESSORS, &nCPU, sizeof(nCPU));
	if(apiretRc != 0) nCPU = 1;
	return nCPU;

#elif defined(WIN32)

	SYSTEM_INFO cpuinfo;
	GetSystemInfo(&cpuinfo);
	return cpuinfo.dwNumberOfProcessors;

#endif	/* USE_WINTHREAD */

	return 1;
} /* getNumOfCPU */
#endif

extern void maskFPU_exception(void);

void
initCPU(void)
{
	/* mask control word with 0xFFF2 */
	/* 0001:invalid
	 * 0100:zero divide
	 * 1000:overflow
	 */
	maskFPU_exception();
} /* setupFPU */

/*********************************************************************/
#if 0
// UIֈړ
void
putCPUinfo(void)
{
	unsigned long unit;

	MPGE_getUnitStates(&unit);

	if(unit & tFAMILY4) RO.printf("familiy 4:");
	if(unit & tFAMILY5) RO.printf("familiy 5:");
	if(unit & tFAMILY6) RO.printf("familiy 6(or over):");

	RO.printf("vendor ");
	if(unit & tINTEL) RO.printf("Intel\n");
	else
	if(unit & tAMD  ) RO.printf("AMD\n");
	else
	if(unit & tCYRIX) RO.printf("Cyrix\n");
	else
	if(unit & tIDT  ) RO.printf("IDT\n");
	else RO.printf("unkown\n");

	RO.printf("extended instruction - ");
	if(unit & tMMX ) RO.printf("MMX ");
	if(unit & tEMMX ) RO.printf("MMX2 ");
	if(unit & tCMOV) RO.printf("CMOV ");
	if(unit & t3DN ) RO.printf("3D Now! ");
	if(unit & tE3DN) RO.printf("Ext. 3D Now! ");
	if(unit & MU_tSSE ) RO.printf("SSE ");
	if(unit & MU_tSSE2) RO.printf("SSE2 ");
	RO.printf("\n");
} /* putCPUinfo */
#endif

/*********************************************************************/

extern void setup_choose_table( int unit );	/* defined in takehiro.c */
extern void setup_quantize_xrpow_ISO( int unit ); /* defined in takehiro.c */
extern void setup_fft( int unit );	/* defined in fft.c */
extern void setup_pow075( int unit ); /* defined in gogo.c */

void
setupUNIT(int unit)
{
//	RO.printf("setup functions\n");
	setup_choose_table( unit );
	setup_quantize_xrpow_ISO( unit );
	setup_fft( unit );
	setup_pow075( unit );
} /* setupUNIT */
