#define _GOGO_C_
#include <stddef.h>
#include "../common.h"
#include "../global.h"
#include "../thread.h"

#if defined(WIN32) || defined(__linux__)
	#define USE_BSS
#endif

static void putAlign(void)
{
	printf( "\talign 32\n" );
}

static void putMember( const char *base, const char *name, int offset )
{
	printf( "%%define %s.%s\t\t(%s+%d)\n", base, name, base, offset );
}

static void putOffset( const char *base, const char *name, int offset )
{
	printf( "%%define %s.%s\t\t(%d)\n", base, name, offset );
}

#define OFFSET( base, x ) ( (int)&base.x - (int)&base )

#ifndef offsetof
  #define offsetof(s,m)   (size_t)&(((s *)0)->m)
#endif

#define putMem( base, x ) putMember( #base, #x, OFFSET( base, x ) )
#define putOff( base, x ) putOffset( #base, #x, offsetof( struct base, x ) )

int main( int argc, char *argv[] )
{
	static const char nameTbl[][4] = { "RO", "RW", "" };
	static const int valSize[] = { sizeof(RO) / 4, sizeof(RW) / 4 };
	int i;

	puts( "\t%include \"nasm.cfg\"" );

	puts( "%ifdef DEF_GLOBAL_VARS" );

	/* global mode */
	printf( "\tglobaldef RO, RW\n" );
#ifdef USE_BSS
	printf( "\tbss_segment\n" );
	for( i = 0; i < 2; i++ ){
		putAlign();
		printf( "%s\tresd %d\n", nameTbl[i], valSize[i] );
	}
#else	/* USE_BSS */
	printf( "\tdata_segment\n" );
	for( i = 0; i < 2; i++ ){
		int size;
		size = valSize[i];
		putAlign();
		puts( nameTbl[i] );
		do{
			int j, remain;
			remain = ( size >= 16 ) ? 16 : size;
			printf( "\tdd " );
			for( j = 0; j < remain - 1; j++ ) printf( "0, " );
			printf( "0\n" );
			size -= remain;
		}while( size );
	}
#endif	/* USE_BSS */

	puts( "%else  ; DEF_GLOBAL_VARS" );

	/* extern mode */
	puts( "\texterndef RO, RW\n" );
	
	putMem( RO, nChannel );
	putMem( RO, inpFreqHz );
	putMem( RO, mode_gr );
	putMem( RO, is_mpeg1 );
	putMem( RO, nSample );
	putMem( RO, rateKbps );
	putMem( RO, emphasis );
	putMem( RO, quality );
	putMem( RO, mode );
	putMem( RO, framesize );
	putMem( RO, version );
	putMem( RO, copyright );
	putMem( RO, original );
	putMem( RO, out_samplerate );
	putMem( RO, ATHtype );
	putMem( RO, brate );
	putMem( RO, padding_type );
	putMem( RO, VBR );
	putMem( RO, VBR_q );
	putMem( RO, VBR_min_bitrate_kbps );
	putMem( RO, VBR_max_bitrate_kbps );
	putMem( RO, unit );
	putMem( RO, nCPU );
	putMem( RO, nThread );
	putMem( RO, tl );
	putMem( RO, inFileName );
	putMem( RO, outFileName );
	putMem( RO, userIn );
	putMem( RO, userOut );
	putMem( RO, useVbrTagWithUserOut );
	putMem( RO, debug );
	putMem( RO, silent );
	putMem( RO, printf );
	putMem( RO, lowpassfreq );
	putMem( RO, highpassfreq );
	putMem( RO, lowpasswidth );
	putMem( RO, highpasswidth );
	putMem( RO, lame_encode_frame_init );
	putMem( RO, iteration_init_init );
	putMem( RO, psymodel_init );
	putMem( RO, channels_out );
	putMem( RO, VBR_min_bitrate );
	putMem( RO, VBR_max_bitrate );
	putMem( RO, CBR_bitrate );
	putMem( RO, samplerate_index );
	putMem( RO, lowpass1 );
	putMem( RO, lowpass2 );
	putMem( RO, highpass1 );
	putMem( RO, highpass2 );
	putMem( RO, lowpass_band );
	putMem( RO, highpass_band );
	putMem( RO, lowpass_start_band );
	putMem( RO, lowpass_end_band );
	putMem( RO, highpass_start_band );
	putMem( RO, highpass_end_band );
	putMem( RO, noise_shaping );
	putMem( RO, noise_shaping_amp );
	putMem( RO, noise_shaping_stop );
	putMem( RO, use_best_huffman );
	putMem( RO, frac_SpF );
	putMem( RO, decay );
	putMem( RO, sideinfo_len );
	putMem( RO, amp_lowpass );
	putMem( RO, amp_highpass );
	putMem( RO, scalefac_band );
	putMem( RO, minval );
	putMem( RO, s3_s );
	putMem( RO, s3_l );
	putMem( RO, cw_upper_index );
	putMem( RO, cw_lower_index );
	putMem( RO, window );
	putMem( RO, window_s );
	putMem( RO, w1_l );
	putMem( RO, w2_l );
	putMem( RO, w1_s );
	putMem( RO, w2_s );
	putMem( RO, mld_l );
	putMem( RO, mld_s );
	putMem( RO, bu_l );
	putMem( RO, bo_l );
	putMem( RO, bu_s );
	putMem( RO, bo_s );
	putMem( RO, npart_l );
	putMem( RO, npart_s );
	putMem( RO, npart_l_orig );
	putMem( RO, npart_s_orig );
	putMem( RO, s3ind );
	putMem( RO, s3ind_s );
	putMem( RO, SNR_s );
	putMem( RO, numlines_s );
	putMem( RO, numlines_l );
	putMem( RO, ATH );
	putMem( RO, nZeroStreamSize );
	putMem( RO, TotalFrameSize );
	putMem( RW, subband_buf );
	putMem( RW, sample_buf );
	putMem( RW, fr0 );
	putMem( RW, fr1 );
	putMem( RW, fr2 );
	putMem( RW, fr3 );
	putMem( RW, num_samples_read );
	putMem( RW, num_samples_to_encode );
	putMem( RW, frameNum );
	putMem( RW, totalframes );
	putMem( RW, tot_ener );
	putMem( RW, ms_ratio );
	putMem( RW, slot_lag );
	putMem( RW, OldValue );
	putMem( RW, CurrentStep );
	putMem( RW, bv_scf );
	putMem( RW, sfb21_extra );
	putMem( RW, header );
	putMem( RW, h_ptr );
	putMem( RW, w_ptr );
	putMem( RW, ancillary_flag );
	putMem( RW, bs );
	putMem( RW, ResvSize );
	putMem( RW, ResvMax );
	putMem( RW, main_data_begin );
	putMem( RW, nb_1 );
	putMem( RW, nb_2 );
	putMem( RW, thm );
	putMem( RW, en );
	putMem( RW, ax_sav );
	putMem( RW, bx_sav );
	putMem( RW, rx_sav );
	putMem( RW, cw );
	putMem( RW, pe );
	putMem( RW, ms_ratio_s_old );
	putMem( RW, ms_ratio_l_old );
	putMem( RW, blocktype_old );
	putMem( RW, VBR_seek_table );
	putMem( RW, nVbrNumFrames );


	putOff( ath_t, use_adjust );
	putOff( ath_t, adjust );
	putOff( ath_t, adjust_limit );
	putOff( ath_t, decay );
	putOff( ath_t, l );
	putOff( ath_t, s );
	putOff( ath_t, cb );
	putOff( bit_stream_struc, buf );
	putOff( bit_stream_struc, buf_size );
	putOff( bit_stream_struc, totbit );
	putOff( bit_stream_struc, buf_byte_idx );
	putOff( bit_stream_struc, buf_bit_idx );
	putOff( gogo_thread_data_s, mfbuf );
	putOff( gogo_thread_data_s, sb );
	putOff( gogo_thread_data_s, sb_sample );
	putOff( gogo_thread_data_s, xr );
	putOff( gogo_thread_data_s, xr_sign );
	putOff( gogo_thread_data_s, xrpow );
	putOff( gogo_thread_data_s, work_xrpow );
	putOff( gogo_thread_data_s, xrpow_sum );
	putOff( gogo_thread_data_s, xrpow_max );
	putOff( gogo_thread_data_s, l3_enc );
	putOff( gogo_thread_data_s, scalefac );
	putOff( gogo_thread_data_s, ath_over );
	putOff( gogo_thread_data_s, l3_xmin );
	putOff( gogo_thread_data_s, wsamp_L );
	putOff( gogo_thread_data_s, wsamp_S );
	putOff( gogo_thread_data_s, energy );
	putOff( gogo_thread_data_s, energy_s );
	putOff( gogo_thread_data_s, tot_ener );
	putOff( gogo_thread_data_s, l3_side );
	putOff( gogo_thread_data_s, next );
	putOff( gogo_thread_data_s, unaligned );
	putOff( gogo_thread_data_s, tid );
	putOff( gogo_thread_data_s, padding );
	putOff( gogo_thread_data_s, mode_ext );
	putOff( gogo_thread_data_s, bitrate_index );
	putOff( gogo_thread_data_s, ResvSize );
	putOff( header_buf_t, write_timing );
	putOff( header_buf_t, ptr );
	putOff( header_buf_t, buf );
	putOff( ro_t, nChannel );
	putOff( ro_t, inpFreqHz );
	putOff( ro_t, mode_gr );
	putOff( ro_t, is_mpeg1 );
	putOff( ro_t, nSample );
	putOff( ro_t, rateKbps );
	putOff( ro_t, emphasis );
	putOff( ro_t, quality );
	putOff( ro_t, mode );
	putOff( ro_t, framesize );
	putOff( ro_t, version );
	putOff( ro_t, copyright );
	putOff( ro_t, original );
	putOff( ro_t, out_samplerate );
	putOff( ro_t, ATHtype );
	putOff( ro_t, brate );
	putOff( ro_t, padding_type );
	putOff( ro_t, VBR );
	putOff( ro_t, VBR_q );
	putOff( ro_t, VBR_min_bitrate_kbps );
	putOff( ro_t, VBR_max_bitrate_kbps );
	putOff( ro_t, unit );
	putOff( ro_t, nCPU );
	putOff( ro_t, nThread );
	putOff( ro_t, tl );
	putOff( ro_t, inFileName );
	putOff( ro_t, outFileName );
	putOff( ro_t, userIn );
	putOff( ro_t, userOut );
	putOff( ro_t, useVbrTagWithUserOut );
	putOff( ro_t, debug );
	putOff( ro_t, silent );
	putOff( ro_t, printf );
	putOff( ro_t, lowpassfreq );
	putOff( ro_t, highpassfreq );
	putOff( ro_t, lowpasswidth );
	putOff( ro_t, highpasswidth );
	putOff( ro_t, lame_encode_frame_init );
	putOff( ro_t, iteration_init_init );
	putOff( ro_t, psymodel_init );
	putOff( ro_t, channels_out );
	putOff( ro_t, VBR_min_bitrate );
	putOff( ro_t, VBR_max_bitrate );
	putOff( ro_t, CBR_bitrate );
	putOff( ro_t, samplerate_index );
	putOff( ro_t, lowpass1 );
	putOff( ro_t, lowpass2 );
	putOff( ro_t, highpass1 );
	putOff( ro_t, highpass2 );
	putOff( ro_t, lowpass_band );
	putOff( ro_t, highpass_band );
	putOff( ro_t, lowpass_start_band );
	putOff( ro_t, lowpass_end_band );
	putOff( ro_t, highpass_start_band );
	putOff( ro_t, highpass_end_band );
	putOff( ro_t, noise_shaping );
	putOff( ro_t, noise_shaping_amp );
	putOff( ro_t, noise_shaping_stop );
	putOff( ro_t, use_best_huffman );
	putOff( ro_t, frac_SpF );
	putOff( ro_t, decay );
	putOff( ro_t, sideinfo_len );
	putOff( ro_t, amp_lowpass );
	putOff( ro_t, amp_highpass );
	putOff( ro_t, scalefac_band );
	putOff( ro_t, minval );
	putOff( ro_t, s3_s );
	putOff( ro_t, s3_l );
	putOff( ro_t, cw_upper_index );
	putOff( ro_t, cw_lower_index );
	putOff( ro_t, window );
	putOff( ro_t, window_s );
	putOff( ro_t, w1_l );
	putOff( ro_t, w2_l );
	putOff( ro_t, w1_s );
	putOff( ro_t, w2_s );
	putOff( ro_t, mld_l );
	putOff( ro_t, mld_s );
	putOff( ro_t, bu_l );
	putOff( ro_t, bo_l );
	putOff( ro_t, bu_s );
	putOff( ro_t, bo_s );
	putOff( ro_t, npart_l );
	putOff( ro_t, npart_s );
	putOff( ro_t, npart_l_orig );
	putOff( ro_t, npart_s_orig );
	putOff( ro_t, s3ind );
	putOff( ro_t, s3ind_s );
	putOff( ro_t, SNR_s );
	putOff( ro_t, numlines_s );
	putOff( ro_t, numlines_l );
	putOff( ro_t, ATH );
	putOff( ro_t, nZeroStreamSize );
	putOff( ro_t, TotalFrameSize );
	putOff( rw_t, subband_buf );
	putOff( rw_t, sample_buf );
	putOff( rw_t, fr0 );
	putOff( rw_t, fr1 );
	putOff( rw_t, fr2 );
	putOff( rw_t, fr3 );
	putOff( rw_t, num_samples_read );
	putOff( rw_t, num_samples_to_encode );
	putOff( rw_t, frameNum );
	putOff( rw_t, totalframes );
	putOff( rw_t, tot_ener );
	putOff( rw_t, ms_ratio );
	putOff( rw_t, slot_lag );
	putOff( rw_t, OldValue );
	putOff( rw_t, CurrentStep );
	putOff( rw_t, bv_scf );
	putOff( rw_t, sfb21_extra );
	putOff( rw_t, header );
	putOff( rw_t, h_ptr );
	putOff( rw_t, w_ptr );
	putOff( rw_t, ancillary_flag );
	putOff( rw_t, bs );
	putOff( rw_t, ResvSize );
	putOff( rw_t, ResvMax );
	putOff( rw_t, main_data_begin );
	putOff( rw_t, nb_1 );
	putOff( rw_t, nb_2 );
	putOff( rw_t, thm );
	putOff( rw_t, en );
	putOff( rw_t, ax_sav );
	putOff( rw_t, bx_sav );
	putOff( rw_t, rx_sav );
	putOff( rw_t, cw );
	putOff( rw_t, pe );
	putOff( rw_t, ms_ratio_s_old );
	putOff( rw_t, ms_ratio_l_old );
	putOff( rw_t, blocktype_old );
	putOff( rw_t, VBR_seek_table );
	putOff( rw_t, nVbrNumFrames );
	putOff( vbr_seek_info_t, sum );
	putOff( vbr_seek_info_t, seen );
	putOff( vbr_seek_info_t, want );
	putOff( vbr_seek_info_t, pos );
	putOff( vbr_seek_info_t, size );
	putOff( vbr_seek_info_t, bag );


	puts( "%endif ; DEF_GLOBAL_VARS" );
	puts( "\tend\n" );
	return 0;
}
