/*
 *	GOGO-no-coda
 *	Copyright(C) 2001 gogo-developer
 */

#if	defined(__linux__)
#define	USE_ITIMER
#include <unistd.h>
#include <sys/time.h>
#include <signal.h>
#  define	min(x,y)	(((x)<(y))?(x):(y))
#endif
#if defined(WIN32)
#include <windows.h>
#endif

#include <stdio.h>
#include <string.h>
#include <time.h>
#include "gogo.h"

#ifndef NOERR
	#define NOERR 0
	#define ERR 1
#endif
#ifndef	TRUE
#  define	TRUE 1
#endif
#ifndef	FALSE
#  define	FALSE 0
#endif

static void usage(void);
static void opening(void);
static void putConfig(void);

static int getOpt(int argc, char *argv[]);
static char *getErrMsg(MERET code);

static int	silent, debug;

// UIŖʓ|ȂȂ񂩂ړB
#define isKANJI1(x) ((unsigned char)((x^0x20)-0xA1)<=0x3B)

#if defined(WIN32) || defined(__HIGHC__)
	#define FILE_SEP '\\'
	#define JAPANESE
#else
	#define FILE_SEP '/'
#endif


#ifdef WIN32
	#define MAX_FILE_LEN 260
#elif BeOS
	#define MAX_FILE_LEN 1024
#else
	#define MAX_FILE_LEN 256
#endif

static int convertToLongFileName(char *fname, const unsigned int maxLen)
{
#if !defined(WIN32)
        return NOERR;
#else
        LPHANDLE hdl;
        WIN32_FIND_DATA info;
        int i, j;
        hdl = FindFirstFile(fname, &info);
        if(hdl == INVALID_HANDLE_VALUE) return ERR;
        if(!*info.cFileName){
                FindClose(hdl);
                return ERR;
        }

        i = (fname[1] == ':') ? 2 : 0;  /* for drive */
        j = i - 1;
        while(i < (int)strlen(fname)){
#if defined(JAPANESE)
                if(isKANJI1(fname[i])){
                        i++;
                }else
#endif
                {
                        if(fname[i] == FILE_SEP) j = i;
                }
                i++;
        }
        fname[j+1] = '\0';       /* patho */
        if(strlen(fname) + strlen(info.cFileName) >= maxLen) return ERR;
        strcat(fname, info.cFileName);
//      if( debug ) fprintf( stderr, "long file name : %s\n", fname );
        FindClose(hdl);
        return NOERR;
#endif /* WIN32 */
} /* convertToLongFileName */
#undef isKANJI1

/* change suffix of src to mp3  */
/* suppose sizeof(dest)>=sizeof(src) */
static int
changeSuffix(char *dest, const char *src, const unsigned int maxLen)
{
        char *p;
        unsigned int i, len;
        len = strlen(src);
        if(len >= maxLen) return ERR;
        strcpy(dest, src);
        p = &dest[len - 1];
        len = min(len, 4); /* if the len of suffix < 4 then replace it with "mp3" */
        for(i = 0; i< len; i++, p--){
                if(*p == '\\' || *p == '/')break; /* no suffix */
                if(*p == '.'){
                        *p = '\0';
                        break;
                }
        }
        if(strlen(dest) + 4 >= maxLen) return ERR;
        strcat(dest, ".mp3");
        return NOERR;
} /* changeSuffix */

/*********************************************************************/

void
putCPUinfo(void)
{
        unsigned long unit;

	if(silent) return;

    MPGE_getUnitStates(&unit);

    if(unit & MU_tFAMILY4) fprintf( stderr, "familiy 4:");
    if(unit & MU_tFAMILY5) fprintf( stderr, "familiy 5:");
    if(unit & MU_tFAMILY6) fprintf( stderr, "familiy 6(or over):");

    fprintf( stderr, "vendor ");
    if(unit & MU_tINTEL) fprintf( stderr, "Intel\n");
    else
    if(unit & MU_tAMD  ) fprintf( stderr, "AMD\n");
    else
    if(unit & MU_tCYRIX) fprintf( stderr, "Cyrix\n");
    else
    if(unit & MU_tIDT  ) fprintf( stderr, "IDT\n");
    else fprintf( stderr, "unkown\n");

    fprintf( stderr, "extended instruction - ");
    if(unit & MU_tMMX ) fprintf( stderr, "MMX ");
    if(unit & MU_tEMMX ) fprintf( stderr, "MMX2 ");
    if(unit & MU_tCMOV) fprintf( stderr, "CMOV ");
    if(unit & MU_t3DN ) fprintf( stderr, "3D Now! ");
    if(unit & MU_tE3DN) fprintf( stderr, "Ext. 3D Now! ");
    if(unit & MU_tSSE ) fprintf( stderr, "SSE ");
    if(unit & MU_tSSE2) fprintf( stderr, "SSE2 ");
    fprintf( stderr, "\n");

} /* putCPUinfo */

/******************   for report   ******************/
static unsigned int totalframes, frameNum, freqHz, samplesPerFrame;
#ifdef	USE_ITIMER
static struct timeval   ElapsedTime;
static volatile struct itimerval itimer;
static struct sigaction new_sig, old_sig;
#elif	defined( USE_TTIMER )
#include "global.h"
#include "thread.h"
static void *			reportThread(gogo_thread_data *tl);
static volatile	clock_t startTime, curTime, beforeTime;
static volatile	char	bTimerEnabled;
static gogo_thread		gThreadShow;
#else
static clock_t startTime, curTime, beforeTime;
#endif



// ASMłgH
static void
write_time_C(int fd, long elapsed_sec, long elapsed_usec, unsigned frame_num,
        unsigned tot_frame, unsigned sps, unsigned spf)
{
	char	buf[1024];
	double	dclock, frame_per, total_sec;
	int	eh,em,es,ems, h,m,s,ms, t;

	dclock = elapsed_sec + elapsed_usec*0.000001;
	total_sec = dclock * tot_frame / frame_num;
	frame_per = 100.0*(double)frame_num/(double)tot_frame;

	t = (int)(total_sec * 100.0);
	ms = t % 100; t /= 100;
	s  = t % 60 ; t /= 60;
	m  = t % 60 ; t /= 60;
	h  = t;

	t = (int)((total_sec-dclock) * 100.0);
	ems = t % 100; t /= 100;
	es  = t % 60 ; t /= 60;
	em  = t % 60 ; t /= 60;
	eh  = t;

	sprintf(buf, "{%7d/%7d} %4.1f%% (%6.2fx)  "
        "re:[%02d:%02d:%02d.%02d] to:[%02d:%02d:%02d.%02d]\r",
	frame_num, tot_frame, frame_per,
	(frame_num * (double)spf) / ((double)sps * dclock),
	eh,em,es,ems, h,m,s,ms);
	write(fd, buf, strlen(buf));
}

// if(n == 0) ̏ꍇ32t[ƕ\B
// VOinhƂČĂ΂ꂽꍇ(n != 0)
static void
reportDsp(int n)
{
#ifdef	USE_ITIMER
	ElapsedTime.tv_sec += itimer.it_interval.tv_sec;
	ElapsedTime.tv_usec += itimer.it_interval.tv_usec;
	ElapsedTime.tv_sec += ElapsedTime.tv_usec/1000000;
	ElapsedTime.tv_usec = ElapsedTime.tv_usec%1000000;
	MPGE_getConfigure(MG_FRAME, &frameNum);
	if(totalframes < frameNum) MPGE_getConfigure(MG_COUNT_FRAME, &totalframes);
	write_time_C(fileno(stderr), ElapsedTime.tv_sec, ElapsedTime.tv_usec,
		frameNum, totalframes, freqHz, samplesPerFrame);
#else
	if(silent) return;
	if((n == 0) && (frameNum & 31) != 0) return;

	curTime = clock();
	if((n == 0) && (curTime - beforeTime < CLOCKS_PER_SEC / 5 )) return;
	MPGE_getConfigure(MG_FRAME, &frameNum);
	if(totalframes < frameNum) MPGE_getConfigure(MG_COUNT_FRAME, &totalframes);
	beforeTime = curTime;
	curTime -= startTime;
	write_time_C(fileno(stdout), curTime/CLOCKS_PER_SEC,
		(curTime%CLOCKS_PER_SEC)*(1000000L/CLOCKS_PER_SEC),
		frameNum, totalframes, freqHz, samplesPerFrame);
#endif
}

static void
reportInit(void)
{
#ifdef USE_ITIMER
	if(!silent){
        	ElapsedTime.tv_sec = ElapsedTime.tv_usec = 0;
        	itimer.it_interval.tv_sec = 0;
        	itimer.it_interval.tv_usec = 200000;    /* XV = 200~b */
        	itimer.it_value = itimer.it_interval;
        	memset(&new_sig, 0, sizeof(struct sigaction));
        	new_sig.sa_handler = reportDsp;
        	new_sig.sa_flags = SA_RESTART;
        	sigaction(SIGALRM, &new_sig, &old_sig);
        	setitimer(ITIMER_REAL, &itimer, NULL);
	}
#else
#if		defined(USE_TTIMER)
	bTimerEnabled	=	TRUE;
	gogo_create_thread( &gThreadShow, reportThread, NULL );
#endif
	startTime = clock();
	beforeTime = startTime;
#endif
	MPGE_getConfigure(MG_OUTFREQ, &freqHz);
	MPGE_getConfigure(MG_NUM_OF_SAMPLES, &samplesPerFrame);
	totalframes = 0;
	frameNum = 0;
}

static void
reportTerm(void)
{
#ifdef USE_ITIMER
	if(!silent){
        	itimer.it_interval.tv_sec = 0;
        	itimer.it_interval.tv_usec = 0; /* stop */
        	itimer.it_value = itimer.it_interval;
        	setitimer(ITIMER_REAL, &itimer, NULL);
        	sigaction(SIGALRM, &old_sig, NULL);
	}
#endif
#if		defined(USE_TTIMER)
	bTimerEnabled	=	FALSE;
	gogo_wait_terminatethread( &gThreadShow );
#endif
	reportDsp(1);
	fputc('\n', stderr);
	fflush(stderr);
}

#if		defined( USE_TTIMER )
static void *
reportThread(gogo_thread_data *tl)
{
	while( bTimerEnabled ){
		Sleep( 200 );
		reportDsp( 1 );
	}
	return	NULL;
}
#endif
int
main(int argc, char *argv[])
{
	MERET ret;

	if(MPGE_initializeWork() == ME_NOFPU) return ERR;

	opening();
	if(argc <= 1 || getOpt(argc, argv) == ERR){
		usage();
		MPGE_endCoder();
		return ERR;
	}

	ret = MPGE_detectConfigure();
	if(ret != ME_NOERR){
		if(!silent) fprintf(stderr, "%s\n", getErrMsg(ret));
		return ERR;
	}

	putConfig();

	reportInit();	/* report progress */

#if		defined(USE_ITIMER) || (defined(_CONSOLE) && defined(WIN32) && defined( USE_TTIMER ))
	ret = MPGE_processTrack();
	if(ret != ME_NOERR){
		if(!silent) fprintf(stderr, "%s\n", getErrMsg(ret));
	}
#else
	for(;;){
		ret = MPGE_processFrame();
		if(ret == ME_EMPTYSTREAM) break;
		if(ret != ME_NOERR){
			if(!silent) fprintf(stderr, "%s\n", getErrMsg(ret));
			break;
		}
		frameNum++;
		reportDsp(0);
	} /* main loop */
#endif
	reportTerm();

	ret = MPGE_closeCoder();
	if(ret != ME_NOERR) if(!silent) fprintf(stderr, "%s\n", getErrMsg(ret));

	MPGE_endCoder();
	clkput();
	return NOERR;
} /* main */

/*************************** option analysis **********************************/
/* no option argument */
static int
tNONE(UPARAM *para, const char *arg)
{
	*para = TRUE;
	return 0;	/* stop, no error */
}

/* integer */
static int
tINT(UPARAM *para, const char *arg)
{
	char *endptr;

	*para = (UPARAM)strtol(arg, &endptr, 10);
	if(*endptr) return -1;	/* stop, error */
	return 1;		/* continue */
}

/* (int)(float_para * 1000 + 0.5) */
static int
tX1000(UPARAM *para, const char *arg)
{
	char *endptr;

	*para = (UPARAM)(strtod(arg, &endptr) * 1000.0 + 0.5);
	if(*endptr) return -1;	/* stop, error */
	return 1;		/* continue */
}

/* on/off */
static int
tBOOL(UPARAM *para, const char *arg)
{
	if(!strcmp(arg, "on")){
		*para = TRUE;
		return 1;	/* continue */
	}else if(!strcmp(arg, "off")){
		*para = FALSE;
		return 1;	/* continue */
	}
	return -1;		/* stop, error */
}

/* pointer of string */
static int
tPTR(UPARAM *para, const char *arg)
{
	*para = (UPARAM)arg;
	return 1;		/* continue */
}

/* igonore(unsupported?) */
static int
tIGNORE(UPARAM *para, const char *arg)
{
	return -1;		/* stop, error */
}

typedef struct{
	char cmd[8];
	UPARAM para;
} cmd_tbl_t;

/* selection */
static int
tM(UPARAM *para, const char *arg)
{
	int	i;
	static cmd_tbl_t cmd_tbl[] = {
		{"m", MC_MODE_MONO},
		{"s", MC_MODE_STEREO},
		{"j", MC_MODE_JOINT},
		{"f", MC_MODE_MSSTEREO},
		{"d", MC_MODE_DUALCHANNEL}
	};

	for(i = 0; i < (sizeof cmd_tbl)/sizeof(cmd_tbl_t); i++){
		if(strcmp(cmd_tbl[i].cmd, arg) == 0){
			*para = (UPARAM)cmd_tbl[i].para;
			return 1;	/* continue */
		}
	}
	return -1;			/* stop, error */
}

/* selection */
static int
tEMP(UPARAM *para, const char *arg)
{
	int	i;
	static cmd_tbl_t cmd_tbl[] = {
		{"n", MC_EMP_NONE},
		{"5", MC_EMP_5015MS},
		{"c", MC_EMP_CCITT}
	};

	for(i = 0; i < (sizeof cmd_tbl)/sizeof(cmd_tbl_t); i++){
		if(strcmp(cmd_tbl[i].cmd, arg) == 0){
			*para = (UPARAM)cmd_tbl[i].para;
			return 1;	/* continue */
		}
	}
	return -1;			/* stop, error */
}


/* selection */
static int
tRIF(UPARAM *para, const char *arg)
{
	int	i;
	static cmd_tbl_t cmd_tbl[] = {
		{"wave", MC_OUTPUT_RIFF_WAVE},
		{"rmp",  MC_OUTPUT_RIFF_RMP}
	};

	for(i = 0; i < (sizeof cmd_tbl)/sizeof(cmd_tbl_t); i++){
		if(strcmp(cmd_tbl[i].cmd, arg) == 0){
			*para = (UPARAM)cmd_tbl[i].para;
			return 1;	/* continue */
		}
	}
	return -1;			/* stop, error */
}


#if defined(WIN32)
/* selection */
static int
tPRI(UPARAM *para, const char *arg)
{
	int	i;
	static cmd_tbl_t cmd_tbl[] = {
		{"highest", THREAD_PRIORITY_TIME_CRITICAL},
		{"high",    THREAD_PRIORITY_HIGHEST},
		{"normal",  THREAD_PRIORITY_NORMAL},
		{"low",     THREAD_PRIORITY_BELOW_NORMAL},
		{"lowest",  THREAD_PRIORITY_IDLE}
	};

	for(i = 0; i < (sizeof cmd_tbl)/sizeof(cmd_tbl_t); i++){
		if(strcmp(cmd_tbl[i].cmd, arg) == 0){
			*para = (UPARAM)cmd_tbl[i].para;
			return 1;	/* continue */
		}
	}
	return -1;			/* stop, error */
}
#endif

enum{
	CON_DEBUG = 1,
	CON_SILENT,
	CON_TEST,
	CON_DELETE,
	CON_LANG
};

typedef struct{
	char name[12];
	int (*type[2])(UPARAM *, const char *);
	UPARAM mode; /* != 0 */
} opt_t;

/* table of options for only console */
static opt_t optTblCON[] = {
	{"debug",	{tNONE,  tNONE},  CON_DEBUG},
	{"silent",	{tNONE,  tNONE},  CON_SILENT},
	{"test",	{tNONE,  tNONE},  CON_TEST},
	{"delete",	{tNONE,  tNONE},  CON_DELETE},
	{"lang",	{tPTR,   tNONE},  CON_LANG},
	{"",		{tIGNORE,tIGNORE},	0}
};

/* table of options for GOGO-API */
static opt_t optTblDLL[] = {
	{"m",		{tM,     tNONE}, MC_ENCODEMODE},
//	{"bswap",	{tNONE,  tNONE},  MC_BYTE_SWAP},
	{"b",		{tINT,   tNONE},  MC_BITRATE},
//	{"br",		{tIGNORE,tNONE},  0},
	{"s",		{tX1000, tNONE},  MC_INPFREQ},
	{"d",		{tX1000, tNONE},  MC_OUTFREQ},
	{"emp",		{tEMP,   tNONE}, MC_EMPHASIS},
	{"mmx",		{tBOOL,  tNONE},  MC_USEMMX},
	{"3dn",		{tBOOL,  tNONE},  MC_USE3DNOW},
	{"sse",		{tBOOL,  tNONE},  MC_USEKNI},
	{"kni",		{tBOOL,  tNONE},  MC_USEKNI},
	{"e3dn",	{tBOOL,  tNONE},  MC_USEE3DNOW},
	{"lpf",		{tBOOL,  tNONE},  MC_USELPF16},
	{"th",		{tINT,   tINT},   MC_MSTHRESHOLD},
	{"i",		{tNONE,  tNONE},  MC_VERIFY},
	{"cpu",		{tINT,   tNONE},  MC_CPU},
#if defined(BeOS)
	{"priority",	{tINT,   tNONE},  MC_THREAD_PRIORITY},
	{"readthread",	{tINT,   tNONE},  MC_READTHREAD_PRIORITY},
#elif defined(WIN32)
	{"priority",	{tPRI,   tNONE}, MC_THREAD_PRIORITY},
#endif
	{"spc1",	{tNONE,  tNONE},  MC_USESPC1},
	{"spc2",	{tNONE,  tNONE},  MC_USESPC2},
	{"v",		{tINT,   tNONE},  MC_VBR},
	{"vb",		{tINT,   tINT},   MC_VBRBITRATE},
	{"offset",	{tINT,   tNONE},  MC_STARTOFFSET},
	{"8bit",	{tNONE,  tNONE},  MC_8BIT_PCM},
	{"mono",	{tNONE,  tNONE},  MC_MONO_PCM},
	{"tos",		{tNONE,  tNONE},  MC_TOWNS_SND},
	{"o",		{tPTR,   tNONE},  MC_OUTPUTDIR},
	{"nopsy",	{tNONE,  tNONE},  MC_USEPSY},
	{"riff",	{tRIF,   tNONE}, MC_OUTPUT_FORMAT},
	/* 牺͂ՂߌŐV */
	{"q",		{tINT,   tNONE},  MC_ENCODE_QUALITY},
	{"a",		{tNONE,  tNONE},  MC_ABR},
	{"",		{tIGNORE,tIGNORE},0}
};

static int
setConsoleOpt(UPARAM mode, UPARAM *para)
{
	switch(mode){
	case CON_DEBUG:
		debug = 1;
		break;
	case CON_SILENT:
		silent = 1;
		break;
	default:
	case CON_LANG:
	case CON_TEST:
	case CON_DELETE:
		if(!silent) fprintf(stderr, "not supported\n");
		return ERR;
	}
	return NOERR;
} /* setConsoleOpt */

/* return: the number of used args */
static int
getOpt1(opt_t *tbl, int argc, char *argv[], UPARAM *para, UPARAM *mode)
{
	int i, num, sel, used;

	/* search option in table */
	for(sel = 0; tbl[sel].name[0]; sel++){
		if(!strcmp(*argv + 1, tbl[sel].name)){
			argc--, argv++;
			*mode = tbl[sel].mode;

			para[0] = para[1] = TRUE;
			num = min(2, argc);

			for(i = 0; i < num; i++){
				used = (*tbl[sel].type[i])(para++, *argv++);
				if(used == 0){
					break;
				}else if(used < 0){
					return used;
				}
			} /* for(i = 0;...) */
			return i + 1;
		}
	}
	return -1;
} /* getOpt1 */

static int
getOpt(int argc, char *argv[])
{
	char inName[MAX_FILE_LEN], outName[MAX_FILE_LEN];
	char *inPtr, *outPtr;

	inPtr = outPtr = NULL;
	argc--, argv++;

	while(argc > 0){
		if(**argv == '-'){
			int usedArgNum;
			UPARAM para[2], mode;

			/* for MPGE_setConfigure */
			usedArgNum = getOpt1(optTblDLL, argc, argv, para, &mode);

			if(usedArgNum > 0){
				/* converse(special case) */
				if(mode == MC_USEPSY) para[0] = !para[0];

				if(MPGE_setConfigure(mode, para[0], para[1]) != ME_NOERR) goto OPT_ERR;

				argc -= usedArgNum;
				argv += usedArgNum;
				continue;
			}

			/* for console option */
			usedArgNum = getOpt1(optTblCON, argc, argv, para, &mode);

			if(usedArgNum > 0){
				if(setConsoleOpt(mode, para) == ERR) goto OPT_ERR;
				argc -= usedArgNum;
				argv += usedArgNum;
				continue;
			}
			goto OPT_ERR;
		}else if(!inPtr){
			inPtr = *argv;
		}else if(!outPtr){
			outPtr = *argv;
		}else{
			goto OPT_ERR;
		}
		argc--, argv++;
	} /* while(argc) */

	if(!inPtr || strlen(inPtr) >= MAX_FILE_LEN) return ERR;
	strcpy(inName, inPtr);

	if(convertToLongFileName(inName, MAX_FILE_LEN) == ERR) return ERR;
	MPGE_setConfigure(MC_INPUTFILE, MC_INPDEV_FILE, (UPARAM)inName);

	if(!outPtr){
		if(changeSuffix(outName, inName, MAX_FILE_LEN) == ERR) return ERR;
	}else{
		if(strlen(outPtr) >= MAX_FILE_LEN) return ERR;
		strcpy(outName, outPtr);
	}
	MPGE_setConfigure(MC_OUTPUTFILE, MC_OUTDEV_FILE, (UPARAM)outName);

	return NOERR;
OPT_ERR:;
	if(!silent) fprintf(stderr, "opt err[%s]\n", *argv);
	return ERR;
} /* getOpt */

static void
usage(void)
{
	if(!silent) fprintf(stderr, "no help\n");
}

static void
opening(void)
{
	if(!silent) {
		char			verbuf[ MGV_BUFLENGTH ];
		unsigned long	vercode;
		if( MPGE_getVersion( &vercode, verbuf ) != ME_NOERR )
			verbuf[0] = '\0';
		fprintf(stderr,
			"GOGO-no-coda %s is a mp3 encoder based on lame 3.88,\n"
			"which is distributed under LGPL on http://www.mp3dev.org/mp3/ .\n"
			"See http://member.nifty.ne.jp/~pen/ .\n", verbuf);
	}
}

static char *
getErrMsg(MERET code)
{
	switch(code){
	case ME_NOERR:
	case ME_EMPTYSTREAM:
		return "no error";
	case ME_PARAMERROR:
		return "parameter error";
	case ME_NOFPU:
		return "FPU is not found";
	case ME_INFILE_NOFOUND:
		return "can't open input file";
	case ME_OUTFILE_NOFOUND:
		return "can't open output file";
	case ME_FREQERROR:
		return "illegal input sampling frequency";
	case ME_BITRATEERROR:
	case ME_BITRATE_ERR:
		return "illegal output bitrate";
	case ME_WAVETYPE_ERR:
		return "illegal wave type";
	case ME_CANNOT_SEEK:
		return "can't seek";
	case ME_HALTED:
		return "halt";
	default:
		return "undefined error";
	}
} /* getErrMsg */

static void
putConfig(void)
{
	putCPUinfo();
	if(!silent) fprintf(stderr, "MPEG 1 layer III\n");
#if 0
	int para;
	char *strBuf;

	if( !MPGE_getConfigure( MG_ENCODEMODE, &nParam ) ){
		switch( nParam ){
			case MC_MODE_MONO:
				strBuf = getMsg( MSG_mono );
				break;
			case MC_MODE_STEREO:
				strBuf = getMsg( MSG_stereo );
				break;
			case MC_MODE_JOINT:
				strBuf = getMsg( MSG_j_stereo );
				break;
			case MC_MODE_MSSTEREO:
				strBuf = getMsg( MSG_ms_stereo );
				break;
			case MC_MODE_DUALCHANNEL:
				strBuf = getMsg( MSG_dual );
				break;
			default:
				strBuf = getMsg( MSG_undefined );
		}
		fprintf(stderr,"%s\n", strBuf );
	}

	if( !MPGE_getConfigure( MG_INPFREQ, &nParam ) ){
		fprintf(stderr, "%s=%.1fkHz ", getMsg( MSG_in_freq ), nParam / 1000.0 );
	}
	if( !MPGE_getConfigure( MG_OUTFREQ, &nParam ) ){
		fprintf(stderr, "%s=%.1fkHz ", getMsg( MSG_out_freq ), nParam / 1000.0 );
	}
	if( !MPGE_getConfigure( MG_BITRATE, &nParam ) ){
		fprintf(stderr,"%s=%dkbps\n", getMsg( MSG_bitrate ), nParam );
	}

	if( !MPGE_getConfigure( MG_INPUTFILE, szInName )  &&
	    !MPGE_getConfigure( MG_OUTPUTFILE, szOutName ) ){
		fprintf(stderr, "%s `%s'\n%s `%s'\n", getMsg( MSG_in_name ), (*szInName)?szInName:"stdin",
			getMsg( MSG_out_name ), szOutName);
	}
#endif
} /* putConfig */

